@@ -1,6 +1,26 @@
 fwhm2bw <- function(hfwhm) hfwhm/sqrt(8*log(2))
 bw2fwhm <- function(h) h*sqrt(8*log(2))
 
+getvofh <- function(bw,lkern,wght){
+.Fortran("getvofh",
+         as.double(bw),
+         as.integer(lkern),
+         as.double(wght),
+         vol=double(1),
+         PACKAGE="fmri")$vol
+}
+gethani <- function(x,y,lkern,value,wght,eps=1e-2){
+.Fortran("gethani",
+         as.double(x),
+         as.double(y),
+         as.integer(lkern),
+         as.double(value),
+         as.double(wght),
+         as.double(eps),
+         bw=double(1),
+         PACKAGE="fmri")$bw
+}
+
 Varcor.gauss<-function(h,interv = 1){
 #
 #   Calculates a correction for the variance estimate obtained by (IQRdiff(y)/1.908)^2
@@ -229,3 +249,28 @@ thcorr3D <- function(bw,lag=rep(5,3)){
   dim(scorr) <- lag
   scorr
 }
+
+connect.mask <- function(mask){
+dm <- dim(mask)
+n1 <- dm[1]
+n2 <- dm[2]
+n3 <- dm[3]
+n <- n1*n2*n3
+mask1 <- .Fortran("lconnect",
+                 as.logical(mask),
+                 as.integer(n1),
+                 as.integer(n2),
+                 as.integer(n3),
+                 as.integer((n1+1)/2),
+                 as.integer((n2+1)/2),
+                 as.integer((n3+1)/2),
+                 integer(n),
+                 integer(n),
+                 integer(n),
+                 logical(n),
+                 mask=logical(n),
+                 DUP=FALSE,
+                 PACKAGE="fmri")$mask
+dim(mask1) <- dm
+mask1
+}
\ No newline at end of file
